@@ -103,6 +103,7 @@ typedef struct {
 	TrackerCrawler *crawler;
 	DirectoryRootInfo  *root_info;
 	DirectoryProcessingData *dir_info;
+	GFile *dir_file;
 } EnumeratorData;
 
 static void     crawler_finalize        (GObject         *object);
@@ -201,9 +202,9 @@ tracker_crawler_init (TrackerCrawler *object)
 {
 	TrackerCrawlerPrivate *priv;
 
-	object->private = TRACKER_CRAWLER_GET_PRIVATE (object);
+	object->priv = TRACKER_CRAWLER_GET_PRIVATE (object);
 
-	priv = object->private;
+	priv = object->priv;
 
 	priv->directories = g_queue_new ();
 	priv->cancellable = g_cancellable_new ();
@@ -263,9 +264,18 @@ check_file (TrackerCrawler    *crawler,
             GFile             *file)
 {
 	gboolean use = FALSE;
+	TrackerCrawlerPrivate *priv;
+
+	priv = TRACKER_CRAWLER_GET_PRIVATE (crawler);
 
 	g_signal_emit (crawler, signals[CHECK_FILE], 0, file, &use);
 
+	/* Crawler may have been stopped while waiting for the 'use' value,
+	 * and the DirectoryRootInfo already disposed... */
+	if (!priv->is_running) {
+		return FALSE;
+	}
+
 	info->files_found++;
 
 	if (!use) {
@@ -278,12 +288,21 @@ check_file (TrackerCrawler    *crawler,
 static gboolean
 check_directory (TrackerCrawler    *crawler,
 		 DirectoryRootInfo *info,
-                 GFile             *file)
+		 GFile             *file)
 {
 	gboolean use = FALSE;
+	TrackerCrawlerPrivate *priv;
+
+	priv = TRACKER_CRAWLER_GET_PRIVATE (crawler);
 
 	g_signal_emit (crawler, signals[CHECK_DIRECTORY], 0, file, &use);
 
+	/* Crawler may have been stopped while waiting for the 'use' value,
+	 * and the DirectoryRootInfo already disposed... */
+	if (!priv->is_running) {
+		return FALSE;
+	}
+
 	info->directories_found++;
 
 	if (!use) {
@@ -405,7 +424,7 @@ process_func (gpointer data)
 	gboolean                 stop_idle = FALSE;
 
 	crawler = TRACKER_CRAWLER (data);
-	priv = crawler->private;
+	priv = crawler->priv;
 
 	if (priv->is_paused) {
 		/* Stop the idle func for now until we are unpaused */
@@ -436,7 +455,10 @@ process_func (gpointer data)
 
 			dir_data->was_inspected = TRUE;
 
-			if (iterate) {
+			/* Crawler may have been already stopped while we were waiting for the
+			 *  check_directory return value, and thus we should check if it's
+			 *  running before going on with the iteration */
+			if (priv->is_running && iterate) {
 				/* Directory contents haven't been inspected yet,
 				 * stop this idle function while it's being iterated
 				 */
@@ -456,15 +478,19 @@ process_func (gpointer data)
 			child_data = dir_data->children->data;
 			dir_data->children = g_slist_remove (dir_data->children, child_data);
 
-			if ((child_data->is_dir &&
-			     check_directory (crawler, info, child_data->child)) ||
-			    (!child_data->is_dir &&
-			     check_file (crawler, info, child_data->child))) {
+			if (((child_data->is_dir &&
+			      check_directory (crawler, info, child_data->child)) ||
+			     (!child_data->is_dir &&
+			      check_file (crawler, info, child_data->child))) &&
+			    /* Crawler may have been already stopped while we were waiting for the
+			     *	check_directory or check_file return value, and thus we should
+			     *	 check if it's running before going on */
+			    priv->is_running) {
 				child_node = g_node_prepend_data (dir_data->node,
 								  g_object_ref (child_data->child));
 			}
 
-			if (child_node && child_data->is_dir) {
+			if (priv->is_running && child_node && child_data->is_dir) {
 				DirectoryProcessingData *child_dir_data;
 
 				child_dir_data = directory_processing_data_new (child_node);
@@ -511,16 +537,16 @@ process_func (gpointer data)
 static gboolean
 process_func_start (TrackerCrawler *crawler)
 {
-	if (crawler->private->is_paused) {
+	if (crawler->priv->is_paused) {
 		return FALSE;
 	}
 
-	if (crawler->private->is_finished) {
+	if (crawler->priv->is_finished) {
 		return FALSE;
 	}
 
-	if (crawler->private->idle_id == 0) {
-		crawler->private->idle_id = g_idle_add (process_func, crawler);
+	if (crawler->priv->idle_id == 0) {
+		crawler->priv->idle_id = g_idle_add (process_func, crawler);
 	}
 
 	return TRUE;
@@ -529,9 +555,9 @@ process_func_start (TrackerCrawler *crawler)
 static void
 process_func_stop (TrackerCrawler *crawler)
 {
-	if (crawler->private->idle_id != 0) {
-		g_source_remove (crawler->private->idle_id);
-		crawler->private->idle_id = 0;
+	if (crawler->priv->idle_id != 0) {
+		g_source_remove (crawler->priv->idle_id);
+		crawler->priv->idle_id = 0;
 	}
 }
 
@@ -547,6 +573,9 @@ enumerator_data_new (TrackerCrawler          *crawler,
 	ed->crawler = g_object_ref (crawler);
 	ed->root_info = root_info;
 	ed->dir_info = dir_info;
+	/* Make sure there's always a ref of the GFile while we're
+	 * iterating it */
+	ed->dir_file = g_object_ref (G_FILE (dir_info->node->data));
 
 	return ed;
 }
@@ -581,6 +610,7 @@ enumerator_data_process (EnumeratorData *ed)
 static void
 enumerator_data_free (EnumeratorData *ed)
 {
+	g_object_unref (ed->dir_file);
 	g_object_unref (ed->crawler);
 	g_slice_free (EnumeratorData, ed);
 }
@@ -628,13 +658,13 @@ file_enumerate_next_cb (GObject      *object,
 
 	ed = user_data;
 	crawler = ed->crawler;
-	cancelled = g_cancellable_is_cancelled (crawler->private->cancellable);
+	cancelled = g_cancellable_is_cancelled (crawler->priv->cancellable);
 
 	files = g_file_enumerator_next_files_finish (enumerator,
 	                                             result,
 	                                             &error);
 
-	if (error || !files || !crawler->private->is_running) {
+	if (error || !files || !crawler->priv->is_running) {
 		if (error && !cancelled) {
 			g_critical ("Could not crawl through directory: %s", error->message);
 			g_error_free (error);
@@ -694,7 +724,7 @@ file_enumerate_next (GFileEnumerator *enumerator,
 	g_file_enumerator_next_files_async (enumerator,
 	                                    FILES_GROUP_SIZE,
 	                                    G_PRIORITY_DEFAULT,
-	                                    ed->crawler->private->cancellable,
+	                                    ed->crawler->priv->cancellable,
 	                                    file_enumerate_next_cb,
 	                                    ed);
 }
@@ -714,7 +744,7 @@ file_enumerate_children_cb (GObject      *file,
 	parent = G_FILE (file);
 	ed = (EnumeratorData*) user_data;
 	crawler = ed->crawler;
-	cancelled = g_cancellable_is_cancelled (crawler->private->cancellable);
+	cancelled = g_cancellable_is_cancelled (crawler->priv->cancellable);
 	enumerator = g_file_enumerate_children_finish (parent, result, &error);
 
 	if (!enumerator) {
@@ -745,16 +775,14 @@ file_enumerate_children (TrackerCrawler          *crawler,
 			 DirectoryProcessingData *dir_data)
 {
 	EnumeratorData *ed;
-	GFile *file;
 
-	file = dir_data->node->data;
 	ed = enumerator_data_new (crawler, info, dir_data);
 
-	g_file_enumerate_children_async (file,
+	g_file_enumerate_children_async (ed->dir_file,
 	                                 FILE_ATTRIBUTES,
 	                                 G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
 	                                 G_PRIORITY_LOW,
-	                                 crawler->private->cancellable,
+	                                 crawler->priv->cancellable,
 	                                 file_enumerate_children_cb,
 	                                 ed);
 }
@@ -770,7 +798,7 @@ tracker_crawler_start (TrackerCrawler *crawler,
 	g_return_val_if_fail (TRACKER_IS_CRAWLER (crawler), FALSE);
 	g_return_val_if_fail (G_IS_FILE (file), FALSE);
 
-	priv = crawler->private;
+	priv = crawler->priv;
 
 	if (!g_file_query_exists (file, NULL)) {
 		return FALSE;
@@ -813,7 +841,12 @@ tracker_crawler_stop (TrackerCrawler *crawler)
 
 	g_return_if_fail (TRACKER_IS_CRAWLER (crawler));
 
-	priv = crawler->private;
+	priv = crawler->priv;
+
+	/* If already not running, just ignore */
+	if (!priv->is_running) {
+		return;
+	}
 
 	priv->is_running = FALSE;
 	g_cancellable_cancel (priv->cancellable);
@@ -842,15 +875,15 @@ tracker_crawler_pause (TrackerCrawler *crawler)
 {
 	g_return_if_fail (TRACKER_IS_CRAWLER (crawler));
 
-	crawler->private->is_paused = TRUE;
+	crawler->priv->is_paused = TRUE;
 
-	if (crawler->private->is_running) {
-		g_timer_stop (crawler->private->timer);
+	if (crawler->priv->is_running) {
+		g_timer_stop (crawler->priv->timer);
 		process_func_stop (crawler);
 	}
 
 	g_message ("Crawler is paused, %s",
-	           crawler->private->is_running ? "currently running" : "not running");
+	           crawler->priv->is_running ? "currently running" : "not running");
 }
 
 void
@@ -858,15 +891,15 @@ tracker_crawler_resume (TrackerCrawler *crawler)
 {
 	g_return_if_fail (TRACKER_IS_CRAWLER (crawler));
 
-	crawler->private->is_paused = FALSE;
+	crawler->priv->is_paused = FALSE;
 
-	if (crawler->private->is_running) {
-		g_timer_continue (crawler->private->timer);
+	if (crawler->priv->is_running) {
+		g_timer_continue (crawler->priv->timer);
 		process_func_start (crawler);
 	}
 
 	g_message ("Crawler is resuming, %s",
-	           crawler->private->is_running ? "currently running" : "not running");
+	           crawler->priv->is_running ? "currently running" : "not running");
 }
 
 void
@@ -876,15 +909,15 @@ tracker_crawler_set_throttle (TrackerCrawler *crawler,
 	g_return_if_fail (TRACKER_IS_CRAWLER (crawler));
 
 	throttle = CLAMP (throttle, 0, 1);
-	crawler->private->throttle = throttle;
+	crawler->priv->throttle = throttle;
 
 	/* Update timeouts */
-	if (crawler->private->idle_id != 0) {
+	if (crawler->priv->idle_id != 0) {
 		guint interval, idle_id;
 
-		interval = MAX_TIMEOUT_INTERVAL * crawler->private->throttle;
+		interval = MAX_TIMEOUT_INTERVAL * crawler->priv->throttle;
 
-		g_source_remove (crawler->private->idle_id);
+		g_source_remove (crawler->priv->idle_id);
 
 		if (interval == 0) {
 			idle_id = g_idle_add (process_func, crawler);
@@ -892,6 +925,6 @@ tracker_crawler_set_throttle (TrackerCrawler *crawler,
 			idle_id = g_timeout_add (interval, process_func, crawler);
 		}
 
-		crawler->private->idle_id = idle_id;
+		crawler->priv->idle_id = idle_id;
 	}
 }
